@@ -11,14 +11,24 @@ from uuid import uuid4
 import aioredis
 import boto3
 from aiokafka import AIOKafkaConsumer, AIOKafkaProducer
+from botocore.config import Config
 
 logger = logging.getLogger("aiokafkadaemon")
 
+OPTS_REDIS_QUEUE_KEY = "redis_queue"
+OPTS_REDIS_URL = "redis_url"
+OPTS_REDIS_NAME = "redis_name"
+OPTS_SQS_QUEUE_URL = "sqs_queue_url"
+
+CONTEXT_REDIS_CHANNEL = "redis_channel"
+CONTEXT_REDIS_QUEUE = "redis_queue"
+CONTEXT_SQS_URL = "sqs_queue_url"
+
 
 class Worker:
 
     _sf_client = None
-    _sqs_client = None
+    _sqs_clients = {}
 
     def __init__(
         self,
@@ -50,6 +60,7 @@ class Worker:
             self._producer_topic = consumer_topic
         self._consumer = None
         self._producer = None
+        self._sqs_opts = sqs_opts
         self._exit = False
 
         # Automatically reads environment variables
@@ -59,15 +70,21 @@ class Worker:
         #       AWS_DEFAULT_REGION
         # https://boto3.amazonaws.com/v1/documentation/api/latest/guide/configuration.html#environment-variables
         try:
-            self._sqs_opts = sqs_opts
             self._sf_client = boto3.client("stepfunctions")
-
-            # only initiate here (low footprint), whether to use is implemented in on_run by subclass
-            self._sqs_client = boto3.client("sqs")
         except Exception as e:
             logger.warning(f"Unable to connect to boto3 client: {e}")
 
-        if create_consumer and not self._sqs_opts.get("queue_url"):
+        # optional redis
+        self._redis_opts = redis_opts
+        self._redis = None
+        self._redis_retry = 0
+        self._redis_reconnecting = False
+
+        if (
+            create_consumer
+            and not sqs_opts.get(OPTS_SQS_QUEUE_URL)
+            and not redis_opts.get(OPTS_REDIS_QUEUE_KEY)
+        ):
             self._consumer = Worker.make_consumer(
                 loop, kafka_broker_addr, kafka_group_id, sasl_opts, consumer_opts
             )
@@ -76,12 +93,6 @@ class Worker:
                 loop, kafka_broker_addr, sasl_opts, producer_opts
             )
 
-        # optional redis
-        self._redis_opts = redis_opts
-        self._redis = None
-        self._redis_retry = 0
-        self._redis_reconnecting = False
-
         # legacy code, to be removed in future releases
         if on_run:
             logger.warning(
@@ -163,9 +174,12 @@ class Worker:
             try:
                 if self._redis_opts:
                     self._redis = await aioredis.create_redis_pool(
-                        self._redis_opts["url"], timeout=5
+                        self._redis_opts[OPTS_REDIS_URL], timeout=5
                     )
-
+                    if self._redis_opts.get(OPTS_REDIS_NAME) is not None:
+                        self._redis.client_setname(
+                            f"{self._redis_opts.get(OPTS_REDIS_NAME)}-{self._uuid}"
+                        )
                     logger.warning("Redis connected")
                 else:
                     logger.warning("Redis opts not supplied")
@@ -195,6 +209,20 @@ class Worker:
                 self._redis = None
                 logger.warning("Redis destoryed")
 
+    def sqs_client(self, queue_url):
+        if not queue_url:
+            return None
+
+        region = queue_url.split(".")[1]
+        client = self._sqs_clients.get(region)
+        if not client:
+            config = Config(region_name=region)
+
+            client = boto3.client("sqs", config=config)
+            self._sqs_clients[region] = client
+
+        return client
+
     async def __aenter__(self):
         """
         Async iterator-enter coroutine
@@ -330,16 +358,44 @@ class Worker:
                         partial(
                             self._sf_client.send_task_success,
                             taskToken=step_functions_task_id,
-                            output=result_data.decode("utf-8"),
+                            output=result_data,
                         ),
                     )
         except Exception as e:
             logger.error(f"Unable to send step functions response: {e}")
             return
 
-    async def send_result(self, result, context=None, result_channel_or_topic=None):
+    async def send_sqs_message(
+        self, queue_url, message, wait=True, high_throughput=True
+    ):
+        client = self.sqs_client(queue_url)
+
+        if queue_url.endswith(".fifo"):
+            did = f"{time.time()}"
+            coro = asyncio.get_running_loop().run_in_executor(
+                None,
+                partial(
+                    client.send_message,
+                    QueueUrl=queue_url,
+                    MessageBody=message,
+                    MessageDeduplicationId=did,
+                    MessageGroupId=did if high_throughput else self._uuid,
+                ),
+            )
+        else:
+            coro = asyncio.get_running_loop().run_in_executor(
+                None,
+                partial(client.send_message, QueueUrl=queue_url, MessageBody=message,),
+            )
+
+        if wait:
+            await coro
+        else:
+            asyncio.ensure_future(coro)
+
+    async def send_result(self, result, context=None, result_channel=None):
         context_topic = None
-        result_data = json.dumps(result).encode("utf-8")
+        result_data = json.dumps(result)
 
         if context:
             if context.get("step_functions_task_id") is not None:
@@ -356,7 +412,10 @@ class Worker:
                     headers = [(k, v.encode("utf-8")) for k, v in headers.items()]
 
                 await self._producer.send(
-                    context_topic, result_data, partition=partition, headers=headers
+                    context_topic,
+                    result_data.encode("utf-8"),
+                    partition=partition,
+                    headers=headers,
                 )
 
                 logger.info(
@@ -364,42 +423,42 @@ class Worker:
                     'to topic "{}"'.format(result.get("code", None), context_topic)
                 )
 
-            elif self._redis and context.get("redis_channel"):
-                redis_channel = context.get("redis_channel")
+            if self._redis:
+                if context.get(CONTEXT_REDIS_CHANNEL):
+                    # can be redis pubsub
+                    redis_channel = context.get(CONTEXT_REDIS_CHANNEL)
 
-                await self._redis.publish(redis_channel, result_data)
+                    await self._redis.publish(redis_channel, result_data)
 
-                logger.info(
-                    "pushing result with code {} "
-                    'to redis "{}"'.format(result.get("code", None), redis_channel)
-                )
-
-        # always check the result_channel_or_topic as it is usually used for data persistence
-        if result_channel_or_topic:
-            if result_channel_or_topic.startswith("https://sqs"):
-                if result_channel_or_topic.endswith(".fifo"):
-                    nowait_delete = asyncio.get_running_loop().run_in_executor(
-                        None,
-                        partial(
-                            self._sqs_client.delete_message,
-                            QueueUrl=result_channel_or_topic,
-                            MessageBody=result_data,
-                            MessageDeduplicationId=f"{time.time()}",
-                            MessageGroupId=self._uuid,
-                        ),
+                    logger.info(
+                        "pushing result with code {} "
+                        'to redis "{}"'.format(result.get("code", None), redis_channel)
                     )
+
+                if context.get(CONTEXT_REDIS_QUEUE):
+                    redis_queue = context.get(CONTEXT_REDIS_QUEUE)
+                    await self._redis.publish(redis_queue, result_data)
+
+                    logger.info(
+                        "pushing result with code {} "
+                        'to redis "{}"'.format(result.get("code", None), redis_queue)
+                    )
+
+            if context.get(CONTEXT_SQS_URL):
+                await self.send_sqs_message(context.get(CONTEXT_SQS_URL), result_data)
+
+        # always check the result_channel as it is usually used for data persistence
+        if result_channel:
+            for rc in result_channel.split(","):
+                if rc.startswith("https://sqs"):
+                    await self.send_sqs_message(rc, result_data)
+                elif rc.startswith("redis://"):
+                    channel = rc.split("redis://")[1]
+                    await self._redis.rpush(channel, result_data)
+                elif self._producer and context_topic != result_channel:
+                    await self._producer.send(rc, result_data)
                 else:
-                    nowait_delete = asyncio.get_running_loop().run_in_executor(
-                        None,
-                        partial(
-                            self._sqs_client.delete_message,
-                            QueueUrl=result_channel_or_topic,
-                            MessageBody=result_data,
-                        ),
-                    )
-                asyncio.ensure_future(nowait_delete)
-            elif self._producer and context_topic != result_channel_or_topic:
-                await self._producer.send(result_channel_or_topic, result_data)
+                    logger.error(f"{rc} result channel cannot be understood")
 
         # unify error logging
         self.capture_error(result=result)
